#!/bin/bash

#
# load parameters
#
source config.in

#
# compile the generator of small mults
#
${host_compile}  -c mults.f90 
${host_compile}  -c multrec_gen.f90 
${host_compile}  mults.o multrec_gen.o small_gen.f90 -o small_gen.x


n=1

#
# generate list of loop bounds to generate for the small library
#
postfixes=""
for ((m=1; m<=${dims}; m++))  ; do
    k=$m
 postfixes="$postfixes ${m}"
done

#
# for easy parallelism go via a Makefile
#
rm -f Makefile.small

(
#
# a two stage approach, first compile in parallel, once done,
# execute in parallel
#
printf "all: bench \n\n"

printf "compile: "
for pf in $postfixes ; do
    printf "comp_${pf} "
done
printf "\n\n"

#
# all compile rules
#
for ((m=1; m<=${dims}; m++))  ; do
    k=$m
    printf "comp_${m}:\n"
    printf "\t mkdir -p run_small_${m}\n"
    printf "\t ./small_gen.x ${m} ${n} ${k} ${transpose_flavor} ${data_type}  > run_small_${m}/small_find.f90\n"
    printf "\t cd run_small_${m} ; ${target_compile} small_find.f90 -o small_find.x ${blas_linking} \n\n"
done 

printf "bench: "
for pf in $postfixes ; do
    printf "bench_${pf} "
done
printf "\n\n"

#
# all execute rules
#
for ((m=1; m<=${dims}; m++))  ; do
    k=$m
    printf "bench_${m}: compile\n"
    printf "\t cd run_small_${m} ; ./small_find.x > small_find.out \n\n"
done

) > Makefile.small

#
# execute makefile compiling all variants and executing them
#

make -j $tasks -f Makefile.small all

#
# analyse results finding optimal small mults
#
(
for ((m=1; m<=${dims}; m++))  ; do
    k=$m
    file=run_small_${m}/small_find.out
    res=`tail -n 1 $file`
    echo "$m $n $k $res"
done
) > small_gen_optimal.out
